using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Text.RegularExpressions;

namespace AplikacjaOkienkowa
{
    /// <summary>
    /// Przechowuje list unikatowych sw oraz iloci ich wystpie w tekcie.
    /// </summary>
    public class Slownik
    {
        private readonly List<Slowo> _listaSlow;

        /// <summary>
        /// Ilo unikatowych sw (rozmiar sownika)
        /// </summary>
        public int IloscSlow {
            get { return _listaSlow.Count; }
           
        }

        /// <summary>
        /// Ilo wszystkich sow odnalezionych w tekcie
        /// </summary>
        public int IloscWszystkichSlow { private set; get; }

        public Slownik()
        {
            _listaSlow = new List<Slowo>();
        }

        /// <summary>
        /// Wykorzystanie wzorca "wstrzyknicie zalenoi" w celu odizolowania 
        /// kodu zwizanego z pobieraniem danych (usuga IReader) od kodu przetwarzajcego te dane.
        /// Metoda umieszcza w sowniku wyrazy z treci dostarczonej przez usug IReader.
        /// </summary>

        public void Czytaj(IReader reader)
        {
            string tekst = reader.Read();

            string [] slowa = Regex.Split(tekst, @"[\W\d_]+");
            foreach (var s in slowa)
            {
                DodajSlowo(s);
            }
        }

        /// <summary>
        /// Pierwotna wersja metody Czytaj, ktra nie wykorzystuje interfejsu IReader.
        /// Metoda odczytuje tekst z pliku i umieszcza wyrazy w sowniku.
        /// </summary>
        public void Czytaj(string nazwaPliku)
        {
            FileStream plik = File.Open(nazwaPliku, FileMode.Open, FileAccess.Read);
            var sr = new StreamReader(plik);
            string tekst = sr.ReadToEnd();
            sr.Close();
            plik.Close();

            string[] slowa = Regex.Split(tekst, @"[\W\d_]+");
            foreach (var s in slowa)
            {
                DodajSlowo(s);
            }
        }

        public IEnumerable<Slowo> ListaSlow()
        {
            _listaSlow.Sort((a, b) => String.Compare(a.Tekst, b.Tekst, false,CultureInfo.CurrentCulture));
            return _listaSlow;
        }

        public void DodajSlowo(string slowo)
        {
            if (slowo == null) throw new ArgumentNullException();

            if (slowo.Length == 0) return;

            slowo = slowo.ToLower();

            IloscWszystkichSlow ++;
            
            foreach (var s in _listaSlow)
            {
                if (s.Tekst == slowo)
                {
                    s.Ilosc ++;
                    return;
                }
            }
           _listaSlow.Add(new Slowo(slowo,1) );
        }
    }
}
