﻿using Xunit;
using System;
using System.Diagnostics;
using System.Linq;
using Moq;

namespace AplikacjaOkienkowa.Tests
{
    /// <summary>
    /// Klasa testowa Xunit.net
    /// </summary>
    public class SlownikTests
    {
        [Fact]
        public void SlownikTest()
        {
            var slownik = new Slownik();

            slownik.DodajSlowo("Ala");
            slownik.DodajSlowo("ma");
            slownik.DodajSlowo("kota");
            slownik.DodajSlowo("ALA");
            slownik.DodajSlowo("AlA");
            slownik.DodajSlowo("Ala");

            Assert.Equal(slownik.IloscSlow, 3);
            Assert.Equal(slownik.IloscWszystkichSlow, 6);
        }

        [Fact]
        public void DodajSlowo_ExceptionTest()
        {
            var s = new Slownik();

            Assert.Throws<ArgumentNullException>(() => s.DodajSlowo(null));
        }

        [Theory]
        [InlineData(new[] { "Ala", "ma", "kota", "ALA" }, 3)]
        [InlineData(new[] { "" }, 0)]
        [InlineData(new[] { "", "", "a", "a" }, 1)]
        public void SlownikTheory(string[] s, int x)
        {
            var slownik = new Slownik();

            foreach (var item in s)
            {
                slownik.DodajSlowo(item);
            }

            Assert.Equal(slownik.IloscSlow, x);
        }


        [Fact]
        public void CzytajTest()
        {
            var slownik = new Slownik();
            string tekst = "Ala ma kota, a kot ma Alę!!!";
            int oczekiwanaLiczbaSlow = 6;

            // Utworzenie atrapy za pomocą biblioteki Moq
            var atrapa = new Mock<IReader>();
            atrapa.Setup(x => x.Read()).Returns(tekst);
            slownik.Czytaj(atrapa.Object);

            Assert.Equal(slownik.IloscSlow, oczekiwanaLiczbaSlow);
            foreach (var slowo in slownik.ListaSlow())
            {
                Debug.WriteLine("_" + slowo.Tekst + "_");
            }

            var oczekiwanaSekwencja = new[] { "a", "ala","alę", "kot", "kota", "ma" };
            var slowaWSlowniku = slownik.ListaSlow().Select(x => x.Tekst).ToArray();
            Assert.Equal(oczekiwanaSekwencja, slowaWSlowniku);
        }

        [Fact]
        public void CzytajTest2()
        {
            var slownik = new Slownik();
            var atrapa = new AtrapaReadera();
            int oczekiwanaLiczbaSlow = 6;

            slownik.Czytaj(atrapa);

            Assert.Equal(slownik.IloscSlow, oczekiwanaLiczbaSlow);
            foreach (var slowo in slownik.ListaSlow())
            {
                Debug.WriteLine("_" + slowo.Tekst + "_");
            }

            var oczekiwanaSekwencja = new[] { "a", "ala", "alę", "kot", "kota", "ma" };
            var slowaWSlowniku = slownik.ListaSlow().Select(x => x.Tekst).ToArray();
            Assert.Equal(oczekiwanaSekwencja, slowaWSlowniku);
        }




        [Fact(Skip = "Pomijam test z pewnego powodu")]
        public void SkipTest()
        {
            Assert.True(false, "Ten test zawsze się kończy niepowodzeniem");
        }

    }
}